#!/bin/bash
# Licensed Materials - Property of IBM
#
# 09N6996
#
# (C) Copyright IBM Corp. 2012 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.

#
# Script function
# Called to configure Storage Control mirroring for HA. We do the following steps:
# 1. Read in our disklayout file sc_disklayout.cfg
# 2. For each line in sc_disklayout.cfg
#    a. Make the approprate mirror0 directory
#    b. Copy the current data in that directory path to the mirror0 directory
# 3. Update the fstab file
# 4. For each line in the disk layout file, mount the bind mount
#
# This script takes optional arguments: 
#   LOGFILE=/path_to_logfile for redirecting the command output (default is /dev/null)
#   SC_DISKLAYOUT_FILE=/path_to_disklayout_file (default is /opt/ibm/vmi/data/disklayout.d/sc_disklayout.cfg)

# Default settings, can be overridden by passing in variable in the format variable=argument
LOGFILE=/var/log/SCinst.log 
ACTION=usage
SC_DISKLAYOUT_FILE=/opt/ibm/vmi/data/disklayout.d/sc_disklayout.cfg
TIMESTAMP=$(date +%Y_%m_%d_%H_%M)
# Specifies the temporary location to store the disklayout file during removal so we can update fstab
SC_DISKLAYOUT_FILE_TMPDIR="/tmp/${TIMESTAMP}_$$"
SC_DISKLAYOUT_FILE_TMP="$SC_DISKLAYOUT_FILE_TMPDIR/sc_disklayout.cfg"

# Reads the parameters passed in to this script  
while getopts ":ho:" option_name; do
  case "$option_name" in
    h) ACTION=usage ;;
    o) ACTION="$OPTARG" ;;
  esac
done

# If LOGFILE is set, use the set value otherwise use /dev/null
LOGFILE=${LOGFILE:=/dev/null}

# Return codes, these are here so they can't be overridden on the command prompt
EXIT_SUCCESS=0
ERROR_NO_DISKLAYOUT_FILE=64
ERROR_MIRROR_NOT_CONFIGURED=65

function usage() {
# Prints a usage message for this script 
#
   echo "This script will copy and add the HA mirror configuration for a disklayout file"
   echo "By default, it will use the layout specified in $SC_DISKLAYOUT_FILE"
   echo "Specify -o add to add the layout specified in $SC_DISKLAYOUT_FILE."
   echo "Specify -o check to check to see if the mirror configuration in $SC_DISKLAYOUT_FILE has already been applied."
   echo "Specify -o clean to remove the layout from fstab and unmount but leave behind the layout file."
   echo "Specify -o remove to remove the layout from fstab and unmount and remove the layout file."
   return $EXIT_SUCCESS
}

function add_mirror_dirs() {
# Argument: disklayout_file
# Reads each disklayout line of the format bind:/mirror0/mirror_path:/filesystem_path
#      creates the mirror directory and copies the filesystem data to it
   local disklayout_file=$1
   while read line ; do 
       local line_as_array=(${line//:/ })
       local mirror_path=${line_as_array[1]}
       local filesystem_path=${line_as_array[2]}
       echo "Read disklayout line $line creating mirror directory $mirror_path and copying $filesystem_path contents to it" >> $LOGFILE 2>&1
       mkdir -p $mirror_path >> $LOGFILE 2>&1  
       if [ -d "$filesystem_path" ]; then
          echo "Copying $filesystem_path to  mirror directory $mirror_path " >> $LOGFILE 2>&1
          cp -fr -p $filesystem_path/* $mirror_path >> $LOGFILE 2>&1  
       fi
   done < $disklayout_file
}

function remove_mirror_dirs() {
# Argument: disklayout_file
# Reads each disklayout line of the format bind:/mirror0/mirror_path:/filesystem_path
#      and removes the mirror directory for it
   local disklayout_file=$1
   while read line ; do 
       local line_as_array=(${line//:/ })
       local mirror_path=${line_as_array[1]}
       local filesystem_path=${line_as_array[2]}
       echo "Read disklayout line $line removing mirror directory $mirror_path " >> $LOGFILE 2>&1
       if [ -d "$mirror_path" ]; then
          echo "Removing mirror directory $mirror_path " >> $LOGFILE 2>&1
          rm -fr $mirror_path >> $LOGFILE 2>&1  
       fi
   done < $disklayout_file
}

function mount_filesystem_dirs() {
# Argument: disklayout_file
# Reads each disklayout line of the format bind:/mirror0/mirror_path:/filesystem_path
#      and mounts the filesystem directory for it.
   local disklayout_file=$1
   while read line ; do 
       local line_as_array=(${line//:/ })
       local filesystem_path=${line_as_array[2]}
       echo "Checking mount point for filesystem path $filesystem_path" >> $LOGFILE 2>&1
       if [ ! -d "$filesystem_path" ]; then
         echo "Creating $filesystem_path for mirror configuration because it is not created yet." >> $LOGFILE 2>&1
         mkdir -p $filesystem_path >> $LOGFILE 2>&1  
       fi
       if ! mountpoint -q "$filesystem_path" ; then
         echo "Mounting filesystem path $filesystem_path" >> $LOGFILE 2>&1
         mount $filesystem_path >> $LOGFILE 2>&1  
       else
         echo "Skipped mounting filesystem $filesystem_path because it was already mounted." >> $LOGFILE 2>&1
       fi
   done < $disklayout_file
}

function unmount_filesystem_dirs() {
# Argument: disklayout_file
# Reads each disklayout line of the format bind:/mirror0/mirror_path:/filesystem_path
#      and unmounts the filesystem directory for it.
   local disklayout_file=$1
   while read line ; do 
       local line_as_array=(${line//:/ })
       local filesystem_path=${line_as_array[2]}
       echo "Checking mount points for unmounting of filesystem path $filesystem_path" >> $LOGFILE 2>&1
       if mountpoint -q "$filesystem_path" ; then
         echo "Unmounting filesystem $filesystem_path" >> $LOGFILE 2>&1
         umount $filesystem_path >> $LOGFILE 2>&1  
       else
         echo "Skipped unmounting filesystem $filesystem_path because it was not mounted." >> $LOGFILE 2>&1
       fi
   done < $disklayout_file
}

function check_mirror_configured() {
# Argument: disklayout_file
# Reads each disklayout line of the format bind:/mirror0/mirror_path:/filesystem_path
#      and checks to see whether we've configured the mirror directory for it
#
   local disklayout_file=$1
   local all_configured=true
   while read line ; do 
       local line_as_array=(${line//:/ })
       local mirror_path=${line_as_array[1]}
       local filesystem_path=${line_as_array[2]}
       echo "Checking disklayout line $line mirror directory $mirror_path " >> $LOGFILE 2>&1
       if [ ! -d "$mirror_path" ]; then
          echo "$mirror_path hasn't been configured, will need to configure." >> $LOGFILE 2>&1
          all_configured=false
          break  
       fi     
       echo "Checking disklayout line $line mount point $filesystem_path " >> $LOGFILE 2>&1
       if ! mountpoint -q "$filesystem_path" ; then
          echo "Mountpoint $filesystem_path hasn't been configured, will need to configure." >> $LOGFILE 2>&1
          all_configured=false
          break
       fi
   done < $disklayout_file
   if $all_configured ; then
     echo "Mirror configuration already installed." >> $LOGFILE 2>&1
     return $EXIT_SUCCESS
   else
     echo "Mirror configuration has not already been installed." >> $LOGFILE 2>&1
     return $ERROR_MIRROR_NOT_CONFIGURED
   fi
}
function add() {
# Adds the mirror configuration specified in SC_DISKLAYOUT_FILE
#
   if [ ! -f $SC_DISKLAYOUT_FILE ]; then
      echo "Disklayout file $SC_DISKLAYOUT_FILE doesn't exist, exiting." >> $LOGFILE 2>&1 
      return $ERROR_NO_DISKLAYOUT_FILE 
   fi 
   add_mirror_dirs $SC_DISKLAYOUT_FILE  
   /opt/ibm/vmi/bin/updatefstab -w >> $LOGFILE 2>&1 
   mount_filesystem_dirs $SC_DISKLAYOUT_FILE
   return $EXIT_SUCCESS 
}

function check() {
# Checks the mirror configuration specified in SC_DISKLAYOUT_FILE to see if
#  it has already been applied.
#
   if [ ! -f $SC_DISKLAYOUT_FILE ]; then
      echo "Disklayout file $SC_DISKLAYOUT_FILE doesn't exist, exiting." >> $LOGFILE 2>&1 
      return $ERROR_NO_DISKLAYOUT_FILE 
   fi 
   check_mirror_configured $SC_DISKLAYOUT_FILE  
   return $?
}

function clean() {
# Clean removes the mirror configuration but leaves the disklayout file 
#
   if [ ! -f $SC_DISKLAYOUT_FILE ]; then
      echo "Disklayout file $SC_DISKLAYOUT_FILE doesn't exist, exiting." >> $LOGFILE 2>&1 
      return $ERROR_NO_DISKLAYOUT_FILE 
   fi 
   remove_mirror_dirs $SC_DISKLAYOUT_FILE
   echo "Moving $SC_DISKLAYOUT_FILE to $SC_DISKLAYOUT_FILE_TMP so we can update fstab without reading in $SC_DISKLAYOUT_FILE" >> $LOGFILE 2>&1
   mkdir -p $SC_DISKLAYOUT_FILE_TMPDIR >> $LOGFILE 2>&1  
   mv -f $SC_DISKLAYOUT_FILE $SC_DISKLAYOUT_FILE_TMP >> $LOGFILE 2>&1  
   /opt/ibm/vmi/bin/updatefstab -w >> $LOGFILE 2>&1 
   unmount_filesystem_dirs $SC_DISKLAYOUT_FILE_TMP
   echo "Moving $SC_DISKLAYOUT_FILE_TMP back to $SC_DISKLAYOUT_FILE" >> $LOGFILE 2>&1
   mv $SC_DISKLAYOUT_FILE_TMP $SC_DISKLAYOUT_FILE >> $LOGFILE 2>&1 
   echo "Removing $SC_DISKLAYOUT_FILE_TMPDIR" >> $LOGFILE 2>&1
   rm -r $SC_DISKLAYOUT_FILE_TMPDIR >> $LOGFILE 2>&1
   return $EXIT_SUCCESS 
}

function remove() {
# Removes the mirror configuration and the disk layout file 
   if [ ! -f $SC_DISKLAYOUT_FILE ]; then
      echo "Disklayout file $SC_DISKLAYOUT_FILE doesn't exist, exiting." >> $LOGFILE 2>&1 
      return $ERROR_NO_DISKLAYOUT_FILE 
   fi 
   clean
   echo "Removing $SC_DISKLAYOUT_FILE" >> $LOGFILE 2>&1
   rm $SC_DISKLAYOUT_FILE  >> $LOGFILE 2>&1 
   return $EXIT_SUCCESS 
}

case $ACTION in
add)
      echo "Adding mirror configuration for $SC_DISKLAYOUT_FILE" >> $LOGFILE 2>&1
      add
      RC=$?
    ;;
check)
      echo "Checking mirror configuration for $SC_DISKLAYOUT_FILE" >> $LOGFILE 2>&1
      check
      RC=$?
    ;;
clean)
      echo "Cleaning mirror configuration for $SC_DISKLAYOUT_FILE" >> $LOGFILE 2>&1
      clean
      RC=$?
    ;;
remove)
      echo "Removing mirror configuration specified in $SC_DISKLAYOUT_FILE then removing $SC_DISKLAYOUT_FILE" >> $LOGFILE 2>&1
      remove
      RC=$?
    ;;
usage)
      echo "Printing usage statement." >> $LOGFILE 2>&1
      usage
      RC=$?
    ;;
*)
      echo "Unknown option, printing usage statement." >> $LOGFILE 2>&1
      usage
      RC=$?
    ;;
esac

echo "Exiting $BASH_SOURCE with return code $RC" >> $LOGFILE 2>&1
exit $RC
